/**
 * EnhProj.e -- $Id: EnhProj.e#5 2009/01/03 10:45:22 chrisant $
 *
 *    Enhanced project utilities.  Initially this is the Slick-C portion of an
 *    enhanced file open dialog, which integrates with a matching EnhProj.dll.
 *    Later this may gain additional abilities.
 *
 * Features:
 *
 *    - The file list is automatically filtered as you type.
 *    - Use the tabs to list Workspace files, Project files, or Any files from
 *      the file system (i.e. starting from the current working directory).
 *
 * Configuration:
 *
 *    Look for "enhproj" below for a list of configuration variables that
 *    control optional behavior.  At this time they can only be set in the
 *    editor via "set-var enhproj_variable_name" at the command line.  At some
 *    point in the future I may hook up a way to set them from the UI.
 *
 * How to filter the list:
 *
 *    - Type a filename prefix to filter the list.
 *    - Type a path\filename prefix to filter by path as well as filename.
 *    - Use * and ? wildcards to filter path or filename using wildcards.
 *    - Type "*foo" to filter the list to files containing "foo" somewhere in
 *      their name.
 *    - Examples:
 *          - Type "foo" to see files that begin with "foo".
 *          - Type "*foo" to see files with "foo" in their name.
 *          - Type "bar\foo" to see files that begin with "foo" in the "bar"
 *            directory.
 *          - Type "bar*\*foo" to see files with "foo" in their name in any
 *            directory that begins with "bar".
 */
/*
 * TODO:
 *
 *    - Hook up UI for choosing options.
 *    - Autocompletion keys for "Any" tab?
 *    - Option to merge the "Any" tab results into the "Project" and
 *      "Workspace" tabs for convenience?
 */

#include "slick.sh"
#pragma option( strict, on )

static _str s_fDllLoaded = false;
int enhproj_ofd_which_list = 2;           // -1 = remember across editor sessions, 0 = dir, 1 = project, 2 = workspace
int enhproj_ofd_remembered_list = 1;      // 0 = dir, 1 = project, 2 = workspace
int enhproj_ofd_sort_by_path = 0;         // 0 = sort by name, 1 = sort by path
int enhproj_ofd_case_sensitive = -1;      // 0 = case insensitive, 1 = case sensitive
int enhproj_ofd_always_substring = 0;     // 0 = default is prefix match, 1 = default is substring match

int enhproj_ofd_x = 0;                    // saved x position of dialog
int enhproj_ofd_y = 0;                    // saved y position of dialog
int enhproj_ofd_cx = 0;                   // saved width of dialog
int enhproj_ofd_cy = 0;                   // saved height of dialog

#ifdef CHRISANT
int enhproj_dbg_dll = 0;                  // 0 = use ship dll, 1 = use debug dll
int enhproj_unload_always = 0;            // 0 = unload at process exit, 1 = unload after closing dialog
#endif

definit()
{
   if ( arg( 1 ) == '' )
      s_fDllLoaded = false;
   if ( enhproj_ofd_case_sensitive < 0 )
      enhproj_ofd_case_sensitive = ( int )( _fpos_case == '' );
}

static void AppendFilesFromCurrentBuffer()
{
   _str filename;

   FlatFile_AllocateFileList( p_Noflines );
   top();
   up();
   while ( !down() )
   {
      get_line( filename );
      FlatFile_AppendFile( filename );
   }
   FlatFile_SortFiles();
}

void EnhProj_InitProjectFiles()
{
   FlatFile_ClearFiles();
   if ( _project_name != '' )
   {
      int file_view_id = 0;
      int orig_view_id = _create_temp_view( file_view_id );

      GetProjectFiles( _project_name, file_view_id,
                       '', null, '', false,
                       true, // absolute paths (may be slow)
                       false, // not listbox format
                       -1, '' );

      AppendFilesFromCurrentBuffer();

      activate_window( orig_view_id );
      _delete_temp_view( file_view_id );
   }
}

void EnhProj_InitWorkspaceFiles()
{
   FlatFile_ClearFiles();
   if ( _workspace_filename != '' )
   {
      int ii;
      _str projects[] = null;
      int file_view_id = 0;
      int orig_view_id = _create_temp_view( file_view_id );

      _GetWorkspaceFiles( _workspace_filename, projects );

      for ( ii = 0; ii < projects._length(); ii++ )
      {
         projects[ii] = _AbsoluteToWorkspace( strip_filename( projects[ii], 'E' ) :+ PRJ_FILE_EXT );
         GetProjectFiles( projects[ii], file_view_id,
                          '', null, '', false,
                          true, // absolute paths (may be slow)
                          false, // not listbox format
                          -1, '' );
      }

      AppendFilesFromCurrentBuffer();

      activate_window( orig_view_id );
      _delete_temp_view( file_view_id );
   }
}

_command void enhOpen() name_info(','VSARG2_READ_ONLY|VSARG2_MARK)
{
   if ( !s_fDllLoaded )
   {
      int status = -1;
      _str s;

#ifdef CHRISANT
      // Construct the path to where a locally built copy of the dll would be.
      s = 'c:\chrisant\projutil\bin.ntx86\';
      if ( enhproj_dbg_dll )
         s :+= 'dbg';
      else
         s :+= 'rtl';
      s :+= '\enhproj.dll';
      status = _dllload( s );
#endif

      if ( status )
      {
         s = slick_path_search( 'enhproj.dll' );
         if ( s._length() )
            status = _dllload( s );
      }

      if ( !status )
         s_fDllLoaded = true;
   }

   int mode = enhproj_ofd_which_list;
   if ( mode < 0 )
      mode = enhproj_ofd_remembered_list;

   FlatFile_SetOptions( enhproj_ofd_sort_by_path,
                        enhproj_ofd_case_sensitive,
                        enhproj_ofd_always_substring );

   FlatFile_SetMode( mode );
   EnhProjDll_SetOfdPos( enhproj_ofd_x, enhproj_ofd_y, enhproj_ofd_cx, enhproj_ofd_cy );
   EnhProjDll_ShowOpenFileDialog();

   int newMode = FlatFile_GetMode();
   if ( enhproj_ofd_remembered_list != newMode ||
        EnhProjDll_IsOfdMoved() )
   {
      enhproj_ofd_remembered_list = newMode;
      enhproj_ofd_x = EnhProjDll_GetOfdX();
      enhproj_ofd_y = EnhProjDll_GetOfdY();
      enhproj_ofd_cx = EnhProjDll_GetOfdCX();
      enhproj_ofd_cy = EnhProjDll_GetOfdCY();
      // Let the editor know that it needs to save the state.
      _config_modify_flags( CFGMODIFY_DEFVAR );
   }

#ifdef CHRISANT
   if ( enhproj_unload_always )
      enhUnload();
#endif
}

_command void enhUnload() name_info(','VSARG2_READ_ONLY|VSARG2_MARK)
{
   if ( s_fDllLoaded )
   {
      /*
       * There is a nasty loader lock deadlock when trying to unload the
       * EnhProj.dll earlier than at process exit time.  The problem is that
       * vsDllExit is the natural point for EnhProj.dll to end its background
       * thread(s), and since the DLL is about to be unloaded it needs to wait
       * for the thread(s) to exit before returning from vsDllExit otherwise
       * the background thread(s) can crash when the DLL is unloaded out from
       * under the running code.  But it can't wait inside vsDllExit because
       * it is inside the loader lock, and the background thread(s) will
       * acquire the loader lock while exiting (see callstack below).  Thus it
       * sets up a deadly embrace.
       *
       * The workaround is for the macro code to be careful about how it
       * unloads the DLL:  the macro explicitly tells the DLL to clean up
       * before unloading it, so that the main thread is not holding the
       * loader lock while the DLL's background thread(s) exit.
       *
       * >  ntdll.dll!_KiFastSystemCallRet@0()
       *    ntdll.dll!_ZwWaitForSingleObject@12()  + 0xc bytes
       *    ntdll.dll!_RtlpWaitOnCriticalSection@8()  + 0xc8 bytes
       *    ntdll.dll!_RtlEnterCriticalSection@4()  - 0x4b84 bytes
       *    ntdll.dll!_LdrGetProcedureAddressEx@20()  + 0xc1 bytes
       *    ntdll.dll!_LdrGetProcedureAddress@16()  + 0x18 bytes
       *    kernel32.dll!_GetProcAddress@8()  + 0x3f bytes
       *    shell32.dll!7650e79d()
       *    [Frames below may be incorrect and/or missing, no symbols loaded for shell32.dll]
       *    kernel32.dll!_HeapFree@12()  + 0x14 bytes
       *    shell32.dll!7650e829()
       *    shell32.dll!76510f1f()
       *    ole32.dll!NotifyInitializeSpies()  + 0x51 bytes
       *    ole32.dll!_CoUninitialize@0()  + 0x36 bytes
       *    ole32.dll!DoThreadSpecificCleanup()  + 0x2edda bytes
       *    ole32.dll!FlsThreadCleanupCallback()  + 0x19 bytes
       *    ntdll.dll!_RtlProcessFlsData@4()  + 0x787 bytes
       *    ntdll.dll!_LdrShutdownThread@0()  + 0x35 bytes
       *    ntdll.dll!_RtlExitUserThread@4()  + 0x2a bytes
       *    msvcrt.dll!__endthreadex()  + 0x23 bytes
       *    msvcrt.dll!__endthreadex()  - 0x25 bytes
       *    msvcrt.dll!__endthreadex()  + 0xf2 bytes
       *    msvcrt.dll!__endthreadex()  + 0xf2 bytes
       *    kernel32.dll!@BaseThreadInitThunk@12()  + 0x12 bytes
       *    ntdll.dll!___RtlUserThreadStart@8()  + 0x27 bytes
       *    ntdll.dll!__RtlUserThreadStart@8()  + 0x1b bytes
       */
      EnhProjDll_StopThreads();
      dunload( "enhproj.dll" );
      s_fDllLoaded = false;
   }
}
