#include "projutil.h"
#include <vsapi.h>
#include <commctrl.h>
#include <richedit.h>
#include "std.h"
#include "str.h"
#include "flatfile.h"
#include "icons.h"
#include "util.h"
#include "opendlg.h"
#include "res.h"

static void
FormatFileInfo( FileInfo const* p, Str& s, Str* ps2 = 0 )
{
	s.Clear();
	if( ps2 )
		ps2->Clear();

	if( FlatFile_GetMode() == 0 )
	{
		s.Printf( "%s", p->m_sFile.Text() );
	}
	else if( !ps2 )
	{
		s.Printf( "%s    (%s)", p->m_sFile.Text(), p->m_sPath.Text() );
	}
	else
	{
		s.Printf( "%s    ", p->m_sFile.Text() );
		ps2->Printf( "(%s)", p->m_sPath.Text() );
	}
}

bool OpenDlg::s_fMoved = false;
int OpenDlg::s_x = 0;
int OpenDlg::s_y = 0;
int OpenDlg::s_cx = 0;
int OpenDlg::s_cy = 0;

OpenDlg::OpenDlg()
	: base()
	, m_himl( 0 )
	, m_fFileListControlInitialized( false )
{
}

OpenDlg::~OpenDlg()
{
	if( m_himl )
		ImageList_Destroy( m_himl );
}

INT_PTR
OpenDlg::DoModal( HWND hwndParent )
{
	return base::DoModal( hwndParent, IDD_OPENFILE, IDR_OPENFILE_ACCEL );
}

void
OpenDlg::SetPos( int const x, int const y, int const cx, int const cy )
{
	s_fMoved = false;
	s_x = x;
	s_y = y;
	s_cx = cx;
	s_cy = cy;
}

void
OpenDlg::UpdatePos( int const x, int const y, int const cx, int const cy )
{
	if( s_x != x || s_y != y || s_cx != cx || s_cy != cy )
	{
		s_fMoved = true;
		s_x = x;
		s_y = y;
		s_cx = cx;
		s_cy = cy;
	}
}

INT_PTR
OpenDlg::DlgProc( UINT const msg, WPARAM const wParam, LPARAM const lParam )
{
	switch( msg )
	{
	ON_MSG( WM_INITDIALOG );
	ON_MSG( WM_COMMAND );
	ON_MSG( WM_SIZE );
	ON_MSG( WM_MOVE );
	ON_MSG( WM_NOTIFY );
	ON_MSG( WM_SETFONT );
	default:
		if( msg == AsyncShellIcon::c_wmIconAvailable )
			return ON_WM_ICONAVAILABLE( wParam, lParam );
	}
	return false;
}

INT_PTR
OpenDlg::ON_WM_INITDIALOG( WPARAM wParam, LPARAM lParam )
{
	// Apply size and position if any is set.
	if( s_cx > 0 && s_cy > 0 )
	{
		SetWindowPos(
			m_hwnd,
			0,
			s_x,
			s_y,
			s_cx,
			s_cy,
			SWP_NOACTIVATE|SWP_NOZORDER );
	}

	// Get the current directory for use in the "Any" tab.
	char sz[MAX_PATH];
	DWORD const cch = GetCurrentDirectory( DimensionOf( sz ), sz );
	if( cch > 0 && cch < DimensionOf( sz ) )
		m_sCwd.Set( sz );
	else
		m_sCwd.Set( "." );
	m_sCwd.Append( "\\" );
	m_sInitFileListPath.Set( m_sCwd );

	// Clear file list in case project/workspace has changed.
	FlatFile_ClearFiles();

	// Initialize controls.  InitFileList also retrieves the project/workspace
	// file list from SlickEdit.
	InitRichEdit( IDC_FILE_EDIT, 128, ENM_KEYEVENTS );
	InitTabControl();
	InitFileList();

	// Layout the controls.
	ON_WM_SIZE( 0, 0 );
	return true;
}

INT_PTR
OpenDlg::ON_WM_COMMAND( WPARAM wParam, LPARAM lParam )
{
	UINT const id = GET_WM_COMMAND_ID( wParam, lParam );
	WORD const code = GET_WM_COMMAND_CMD( wParam, lParam );
	HWND const hwndControl = GET_WM_COMMAND_HWND( wParam, lParam );

	switch( id )
	{
	case IDC_FILE_EDIT:
		switch( code )
		{
		case EN_CHANGE:
			if( FlatFile_GetMode() == 0 && DoesListNeedReinitialized() )
				InitFileList();
			else
				ReloadFileList();
			break;
		}
		break;

	case IDM_OPENFILE_ALLFILES:
	case IDM_OPENFILE_PROJECTFILES:
	case IDM_OPENFILE_WORKSPACEFILES:
		TabCtrl_SetCurSel( GetDlgItem( IDC_LIST_TAB ), id - IDM_OPENFILE_ALLFILES );
		InitFileList();
		break;

	case IDM_EDITUNDO:
	case IDM_EDITCUT:
	case IDM_EDITCOPY:
	case IDM_EDITPASTE:
	case IDM_EDITCLEAR:
	case IDM_EDITSELECTALL:
		HandleEditCommand( hwndControl, id );
		break;

	case IDOK:
		if( TryToOpenFile() )
			EndDialog( true );
		break;
	case IDCANCEL:
		EndDialog( false );
		break;

	default:
		return false;
	}

	return true;
}

INT_PTR
OpenDlg::ON_WM_SIZE( WPARAM wParam, LPARAM lParam )
{
	RECT rc;
	RECT rcClient;
	DeferPosXact dpx;
	LONG yy;

	GetClientRect( m_hwnd, &rcClient );

	dpx.Begin( 10 );

	yy = rcClient.top + 4;
	GetWindowRect( GetDlgItem( IDC_FILE_LABEL ), &rc );
	SetRect( &rc, rcClient.left + 4, yy, rcClient.right - 4, yy + rc.bottom - rc.top );
	dpx.DeferWindowPos( GetDlgItem( IDC_FILE_LABEL ), &rc );

	yy = rc.bottom;
	GetWindowRect( GetDlgItem( IDC_FILE_EDIT ), &rc );
	SetRect( &rc, rcClient.left + 4, yy, rcClient.right - 4, yy + rc.bottom - rc.top );
	dpx.DeferWindowPos( GetDlgItem( IDC_FILE_EDIT ), &rc );

	yy = rc.bottom + 6;
	GetWindowRect( GetDlgItem( IDC_LIST_TAB ), &rc );
	SetRect( &rc, rcClient.left + 4, yy, rcClient.right - 4, yy + rc.bottom - rc.top );
	dpx.DeferWindowPos( GetDlgItem( IDC_LIST_TAB ), &rc );
	//$ BUGBUG: (chrisant) The tab control doesn't line up properly with the listview control...
	//$ todo: (chrisant) I'd like the tab control to go at the bottom, but I can't get the tabs to paint as though they're on the bottom...

	RECT rcWhere;
	GetWindowRect( GetDlgItem( IDC_WHERE_LABEL ), &rcWhere );
	LONG const cyWhere = rcWhere.bottom - rcWhere.top;

	yy = rc.bottom;
	HWND const hwndList = GetDlgItem( IDC_LIST_LIST );
	GetWindowRect( hwndList, &rc );
	SetRect( &rc, rcClient.left + 4, yy, rcClient.right - 4, Max( rcClient.bottom - 4 - cyWhere - 4, yy + LONG( 72 ) ) );
	dpx.DeferWindowPos( hwndList, &rc );

	yy = rc.bottom + 4;
	SetRect( &rc, rcClient.left + 4, yy, rcClient.right - 4, yy + cyWhere );
	dpx.DeferWindowPos( GetDlgItem( IDC_WHERE_LABEL ), &rc );

	dpx.End();

	GetClientRect( hwndList, &rcClient );
	ListView_SetColumnWidth( hwndList, 0, rcClient.right - rcClient.left );

	RememberPos();
	return true;
}

INT_PTR
OpenDlg::ON_WM_MOVE( WPARAM wParam, LPARAM lParam )
{
	RememberPos();
	return true;
}

inline bool
TestKeyState( int vk )
{
	return GetKeyState( vk ) < 0;
}

static WORD
TestKeyStates()
{
	WORD w = 0;
	if( TestKeyState( VK_CONTROL ) ) w |= MOD_CONTROL;
	if( TestKeyState( VK_SHIFT ) ) w |= MOD_SHIFT;
	if( TestKeyState( VK_MENU ) ) w |= MOD_ALT;
	return w;
}

INT_PTR
OpenDlg::ON_WM_NOTIFY( WPARAM wParam, LPARAM lParam )
{
	NMHDR const* const pnm = reinterpret_cast<NMHDR const*>( lParam );

	switch( pnm->code )
	{
	case NM_CUSTOMDRAW:
		if( pnm->idFrom == IDC_LIST_LIST )
		{
			Str s;
			Str s2;
			NMLVCUSTOMDRAW const* const plvcd = reinterpret_cast<NMLVCUSTOMDRAW const*>( lParam );
			switch( plvcd->nmcd.dwDrawStage )
			{
			case CDDS_PREPAINT:
				SetWindowLong( m_hwnd, DWL_MSGRESULT, CDRF_NOTIFYITEMDRAW );
				break;
			case CDDS_ITEMPREPAINT:
				if( plvcd->nmcd.rc.right - plvcd->nmcd.rc.left <= 0 ||
					plvcd->nmcd.rc.bottom - plvcd->nmcd.rc.top <= 0 )
				{
					//$ review: (chrisant) Why does this get called when
					// painting isn't even happening?  Is it part of tooltip
					// calculation or something?
					break;
				}
				FileInfo* const p = FlatFile_GetFile( plvcd->nmcd.dwItemSpec );
				if( p )
					FormatFileInfo( p, s, &s2 );
				//$ review: (chrisant) uItemState is supposed to indicate
				// whether the item is selected, but it's not working for me.
				//bool const fSelected = !!( plvcd->nmcd.uItemState & CDIS_SELECTED );
				bool const fSelected = !!ListView_GetItemState( pnm->hwndFrom, plvcd->nmcd.dwItemSpec, LVIS_SELECTED );
				COLORREF const crText = GetSysColor( fSelected ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT );
				COLORREF const crBack = GetSysColor( fSelected ? COLOR_HIGHLIGHT : COLOR_WINDOW );
				COLORREF const crTextOld = SetTextColor( plvcd->nmcd.hdc, crText );
				COLORREF const crBackOld = SetBkColor( plvcd->nmcd.hdc, crBack );
				int const modeOld = SetBkMode( plvcd->nmcd.hdc, TRANSPARENT );
				int const dxIcon = 4 + ( m_himlSysSmallIcons ? 16 : 0 );
				int xx = 2 + dxIcon;
				if( p && ( p->m_attr & ( FILE_ATTRIBUTE_HIDDEN|FILE_ATTRIBUTE_SYSTEM ) ) )
					SetTextColor( plvcd->nmcd.hdc, MakeBlendedGray( plvcd->nmcd.hdc ) );
				ExtTextOut( plvcd->nmcd.hdc, plvcd->nmcd.rc.left + xx, ( plvcd->nmcd.rc.top + plvcd->nmcd.rc.bottom - m_cyItem ) / 2, ETO_OPAQUE, &plvcd->nmcd.rc, s.Text(), s.Length(), 0 );
				if( m_himlSysSmallIcons )
				{
					if( p->EnsureSysIcon( m_hwnd ) )
						ImageList_Draw( m_himlSysSmallIcons, p->m_iImage, plvcd->nmcd.hdc, plvcd->nmcd.rc.left + xx - dxIcon, ( plvcd->nmcd.rc.top + plvcd->nmcd.rc.bottom - 16 ) / 2, ILD_TRANSPARENT );
				}
				if( s2.Length() )
				{
					SIZE size;
					GetTextExtentPoint32( plvcd->nmcd.hdc, s.Text(), s.Length(), &size );
					COLORREF const crBlended = MakeBlendedGray( plvcd->nmcd.hdc, fSelected ? 0xaa : 0x80, true );
					SetTextColor( plvcd->nmcd.hdc, crBlended );
					ExtTextOut( plvcd->nmcd.hdc, plvcd->nmcd.rc.left + xx + size.cx, ( plvcd->nmcd.rc.top + plvcd->nmcd.rc.bottom - m_cyItem ) / 2, 0, &plvcd->nmcd.rc, s2.Text(), s2.Length(), 0 );
				}
				if( plvcd->nmcd.uItemState & CDIS_FOCUS )
					DrawFocusRect( plvcd->nmcd.hdc, &plvcd->nmcd.rc );
				SetBkMode( plvcd->nmcd.hdc, modeOld );
				SetBkColor( plvcd->nmcd.hdc, crBackOld );
				SetTextColor( plvcd->nmcd.hdc, crTextOld );
				SetWindowLong( m_hwnd, DWL_MSGRESULT, CDRF_SKIPDEFAULT );
				break;
			}
		}
		break;
	case EN_MSGFILTER:
		if( pnm->idFrom == IDC_FILE_EDIT )
		{
			MSGFILTER* const pmf = reinterpret_cast<MSGFILTER*>( lParam );
			if( pmf->msg != WM_KEYDOWN && pmf->msg != WM_KEYUP )
				break;
			bool fIgnoreMessage = true;
			switch( pmf->wParam )
			{
			case VK_UP:
			case VK_DOWN:
			case VK_PRIOR:
			case VK_NEXT:
				SendDlgItemMessage( IDC_LIST_LIST, pmf->msg, pmf->wParam, pmf->lParam );
				break;
			case VK_HOME:
			case VK_END:
				{
					WORD const states = TestKeyStates();
					if( states && states != MOD_SHIFT )
						SendDlgItemMessage( IDC_LIST_LIST, pmf->msg, pmf->wParam, pmf->lParam );
					else
						fIgnoreMessage = false;
				}
				break;
			default:
				fIgnoreMessage = false;
				break;
			}
			if( fIgnoreMessage )
				::SetWindowLong( m_hwnd, DWL_MSGRESULT, 1 );	// Tell caller to ignore the event.
		}
		break;
	case LVN_GETDISPINFO:
		if( pnm->idFrom == IDC_LIST_LIST )
		{
			NMLVDISPINFO* const plvd = reinterpret_cast<NMLVDISPINFO*>( lParam );
			if( plvd->item.mask & LVIF_TEXT )
			{
				FileInfo const* const p = FlatFile_GetFile( plvd->item.iItem );
				if( p )
				{
					m_sToolTip.Clear();
					FormatFileInfo( p, m_sToolTip );

					Assert( plvd->item.cchTextMax );
					lstrcpyn( plvd->item.pszText, m_sToolTip.Text(), plvd->item.cchTextMax );
					//plvd->item.iIndent = 0;
				}
			}
		}
		break;
	case LVN_ITEMACTIVATE:
		if( pnm->idFrom == IDC_LIST_LIST )
		{
#if 0
			NMITEMACTIVATE* const pia = reinterpret_cast<NMITEMACTIVATE*>( lParam );
#endif
			if( TryToOpenFile() )
				EndDialog( true );
		}
		break;
	case TCN_SELCHANGE:
		if( pnm->idFrom == IDC_LIST_TAB )
		{
			InitFileList();
		}
		break;
	case TTN_GETDISPINFO:
		if( pnm->hwndFrom == TabCtrl_GetToolTips( GetDlgItem( IDC_LIST_TAB ) ) )
		{
			NMTTDISPINFO* const pttdi = reinterpret_cast<NMTTDISPINFO*>( lParam );
			char const* psz = "";
			switch( pnm->idFrom )
			{
			case 0:		psz = "List any files (Ctrl+F)"; break;
			case 1:		psz = "List project files (Ctrl+P)"; break;
			case 2:		psz = "List workspace files (Ctrl+W)"; break;
			}
			lstrcpyn( pttdi->szText, psz, DimensionOf( pttdi->szText ) );
		}
		break;
	}

	return true;
}

INT_PTR
OpenDlg::ON_WM_SETFONT( WPARAM wParam, LPARAM lParam )
{
	InitFileListFont( HFONT( wParam ) );
	return false;
}

INT_PTR
OpenDlg::ON_WM_ICONAVAILABLE( WPARAM wParam, LPARAM lParam )
{
	InvalidateRect( GetDlgItem( IDC_LIST_LIST ), 0, false );
	return true;
}

void
OpenDlg::InitTabControl()
{
	static char const* const c_rgTabs[] = { "Any", "Project", "Workspace" };

	HWND const hwndCtl = GetDlgItem( IDC_LIST_TAB );

	if( !m_himl )
		m_himl = ImageList_LoadImage( ResourceModule(), MAKEINTRESOURCE( IDB_IMAGELIST ), 16, 8, RGB( 255, 0, 255 ), IMAGE_BITMAP, LR_CREATEDIBSECTION|LR_LOADTRANSPARENT );

	TabCtrl_SetImageList( hwndCtl, m_himl );

	TCITEM tci = { 0 };
	tci.mask = TCIF_TEXT|TCIF_IMAGE;
	for( unsigned ii = 0; ii < DimensionOf( c_rgTabs ); ii++ )
	{
		tci.iImage = II_BBFILELIST + ii;
		tci.pszText = const_cast<char*>( c_rgTabs[ii] );
		TabCtrl_InsertItem( hwndCtl, ii, &tci );
	}

	TabCtrl_SetCurSel( hwndCtl, FlatFile_GetMode() );
}

void
OpenDlg::InitFileList()
{
	if( !m_fFileListControlInitialized )
	{
		m_fFileListControlInitialized = true;

		RECT rcClient;
		HWND const hwndCtl = GetDlgItem( IDC_LIST_LIST );

		ListView_SetExtendedListViewStyle( hwndCtl, LVS_EX_FULLROWSELECT | LVS_EX_DOUBLEBUFFER | LVS_EX_INFOTIP );
		ThemeExplorerTheme( hwndCtl );
		InitFileListFont();

		GetClientRect( m_hwnd, &rcClient );

		LVCOLUMN lvc = { 0 };
		lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
		lvc.fmt = LVCFMT_LEFT;
		lvc.pszText = "File";
		lvc.cx = rcClient.right - rcClient.left;
		ListView_InsertColumn( hwndCtl, 0, &lvc );
	}

	// Update the file list.

	int const iTab = TabCtrl_GetCurSel( GetDlgItem( IDC_LIST_TAB ) );
	FlatFile_SetMode( iTab );
	switch( FlatFile_GetMode() )
	{
	case 0:
		DoesListNeedReinitialized();
		SetWindowText( GetDlgItem( IDC_WHERE_LABEL ), m_sCwd.Text() );
		FlatFile_ClearFiles();
		if( !FlatFile_AllocateFileList( 8 ) )
		{
			Str s;
			s.Set( m_sInitFileListPath );
			s.Append( "*" );

			WIN32_FIND_DATA fd;
			HANDLE const hFind = FindFirstFile( s.Text(), &fd );
			if( hFind != INVALID_HANDLE_VALUE )
			{
				do
				{
					if( lstrcmp( fd.cFileName, "." ) != 0 )
						FlatFile_AppendFileData( m_sInitFileListPath.Text(), &fd );
				}
				while( FindNextFile( hFind, &fd ) );
				FlatFile_SortFiles();
			}
		}
		break;
	case 1:
	case 2:
		{
			Str s;
			char const* const pszVar = ( FlatFile_GetMode() == 2 ) ? "_workspace_filename" : "_project_name";
			int const index = vsFindIndex( pszVar, VSTYPE_VAR );
			if( index >= 0 )
			{
				VSHVAR const hvar = vsGetVar( index );
				s.Set( GetFileName( vsHvarGetZ( hvar ) ) );
				s.SetEnd( GetExtension( s.Text() ) );
				s.Append( ( FlatFile_GetMode() == 2 ) ? " workspace" : " project" );
			}
			SetWindowText( GetDlgItem( IDC_WHERE_LABEL ), s.Text() );
		}
		if( !FlatFile_Count() )
		{
			char const* const pszProc = ( FlatFile_GetMode() == 2 ) ? "EnhProj_InitWorkspaceFiles" : "EnhProj_InitProjectFiles";
			int const index = vsFindIndex( pszProc, VSTYPE_PROC );
			if( index >= 0 )
				vsCallIndex( 0, index, 0, 0 );
		}
		break;
	}

	// Reload the filtered file list.

	ReloadFileList();
}

void
OpenDlg::InitFileListFont( HFONT hfont )
{
	if( !hfont )
		hfont = HFONT( SendDlgItemMessage( IDC_LIST_LIST, WM_GETFONT, 0, 0 ) );

	TEXTMETRIC tm;
	HDC const hdc = GetDC( m_hwnd );
	HFONT const hfontOld = SelectFont( hdc, hfont );

	GetTextMetrics( hdc, &tm );
	m_cyItem = tm.tmHeight;

	SelectFont( hdc, hfontOld );
	ReleaseDC( m_hwnd, hdc );
}

/**
 * Determine whether the list needs to be reinitialized, or just refiltered.
 *
 * @param pszPath		If no path is specified, then the path in the edit box
 *						is used.
 * @param fCanUpdate	When true, the edit box can be updated.
 *
 * @return bool			Returns true if the list should be re-initialized, or
 *						false if the list just needs to be re-filtered.
 */
bool
OpenDlg::DoesListNeedReinitialized( char const* pszPath, bool fCanUpdate )
{
	Str sPath;
	HWND const hwndEdit = GetDlgItem( IDC_FILE_EDIT );

	if( pszPath )
		sPath.Set( pszPath );
	else
		sPath.GetWindowText( hwndEdit );

	// If the directory component of the new and old paths are the same then
	// the list does not need to be reinitialized.

	Str sNewPath;
	Str sOldPath;
	sNewPath.Set( sPath );
	sOldPath.Set( m_sInitFileListPath );

	char const* const pszNewFile = GetFileName( sNewPath.Text() );
	char const* const pszOldFile = GetFileName( sOldPath.Text() );
	sNewPath.SetEnd( pszNewFile );
	sOldPath.SetEnd( pszOldFile );

	if( sNewPath.EqualI( sOldPath ) )
		return false;

	// Convert to absolute path.  If conversion fails then we're out easy and
	// the list doesn't need to be reinitialized.

	Str sFile;
	char sz[1024];

	sFile.Set( GetFileName( sPath.Text() ) );
	if( sPath.Length() )
		sPath.SetLength( sPath.Length() - sFile.Length() );
	vsFileAbsoluteTo( sz, sPath.Text(), m_sCwd.Text() );

	if( !sz[0] )
		return false;

	// The list needs to be reinitialized.  If we can update the edit box,
	// then do so now.

	m_sInitFileListPath.Set( sz );
	m_sInitFileListPath.SetEnd( GetFileName( m_sInitFileListPath.Text() ) );

	if( fCanUpdate )
	{
		m_sCwd.Set( sz );

		Assert( m_sCwd.Length() );
		Assert( m_sCwd.Text()[m_sCwd.Length() - 1] == '\\' );

		SetWindowText( hwndEdit, sFile.Text() );
		::SendMessage( hwndEdit, EM_SETSEL, sFile.Length(), sFile.Length() );
	}

	return true;
}

void
OpenDlg::ReloadFileList()
{
	RECT rcClient;
	GetClientRect( m_hwnd, &rcClient );

	// Filter the file list.

	Str s;
	s.GetWindowText( GetDlgItem( IDC_FILE_EDIT ) );
	if( FlatFile_GetMode() == 0 )
	{
		Str s2;
		s2.Set( GetFileName( s.Text() ) );
		s.Set( "\\" );
		s.Append( s2 );
	}
	FlatFile_Filter( s.Text() );

	// Get the list and disable redraw until finished.

	HWND const hwndCtl = GetDlgItem( IDC_LIST_LIST );
	::SendMessage( hwndCtl, WM_SETREDRAW, false, 0 );

	// Set count of list items.

	ListView_SetItemCount( hwndCtl, FlatFile_Count() );
	ListView_SetItemState( hwndCtl, 0, LVIS_FOCUSED|LVIS_SELECTED, 0x000f );

	SHFILEINFO shfi;
	m_himlSysSmallIcons = HIMAGELIST( SHGetFileInfo( "x", FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof( shfi ), SHGFI_USEFILEATTRIBUTES|SHGFI_SYSICONINDEX|SHGFI_SMALLICON ) );
	ListView_SetImageList( hwndCtl, m_himlSysSmallIcons, LVSIL_SMALL );

	// Enable redraw and refresh the list.

	::SendMessage( hwndCtl, WM_SETREDRAW, true, 0 );

	GetClientRect( hwndCtl, &rcClient );
	ListView_SetColumnWidth( hwndCtl, 0, rcClient.right - rcClient.left );

	InvalidateRect( hwndCtl, 0, false );
}

bool
OpenDlg::TryToOpenFile()
{
	Str sTyped;
	Str sSelected;

	sTyped.GetWindowText( GetDlgItem( IDC_FILE_EDIT ) );
	sTyped.Trim();

	unsigned const index = ListView_GetNextItem( GetDlgItem( IDC_LIST_LIST ), -1, LVNI_SELECTED );
	FileInfo const* const p = FlatFile_GetFile( index );

	if( p )
	{
		sSelected.Set( p->m_sPath );
		sSelected.Append( p->m_sFile );
	}

	char const* pszCandidate = p ? sSelected.Text() : sTyped.Text();

	if( pszCandidate && *pszCandidate )
	{
		if( FlatFile_GetMode() == 0 )
		{
			bool const fDirFromList = ( p && ( p->m_attr & FILE_ATTRIBUTE_DIRECTORY ) );
			DWORD const attr = !fDirFromList ? GetFileAttributes( pszCandidate ) : 0;
			bool const fDirNotFromList = ( attr != DWORD( -1 ) && ( attr & FILE_ATTRIBUTE_DIRECTORY ) );

			if( fDirNotFromList )
			{
				sSelected.Set( pszCandidate );
				if( sSelected.Length() && sSelected.Text()[sSelected.Length() - 1] != '\\' )
					sSelected.Append( "\\" );
			}

			if( fDirFromList || fDirNotFromList )
			{
				if( DoesListNeedReinitialized( pszCandidate, true ) )
					InitFileList();
				return false;
			}
		}

		Str sEdit;
		sEdit.Printf( "e \"%s\"", pszCandidate );
		int status = vsExecute( 0, sEdit.Text() );
		if( !status || status == NEW_FILE_RC )
			return true;
	}

	return false;
}

void
OpenDlg::RememberPos()
{
	RECT rc;

	// Remember the current position and size.

	GetWindowRect( m_hwnd, &rc );
	UpdatePos( rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top );
}

