#include "projutil.h"
#include <vsapi.h>
#include "std.h"
#include "str.h"
#include "flatfile.h"
#include "icons.h"
#include "opendlg.h"

static HINSTANCE s_hinstRE = 0;

EXTERN_C_BEGIN

/**
 * vsdllinit is called from dllmain.obj which is typically linked in your DLL.
 * This entry point gets called no matter how the dll gets loaded.
 */
void VSAPI
vsDllInit()
{
	OleInitialize( 0 );	// For SHGetFileInfo.

	s_hinstRE = LoadLibrary( "msftedit.dll" );

	Str::InitializeHeap();

	vsDllExport( "void EnhProjDll_SetOfdPos( int x, int y, int cx, int cy )", 0, 0);
	vsDllExport( "int EnhProjDll_IsOfdMoved()", 0, 0);
	vsDllExport( "int EnhProjDll_GetOfdX()", 0, 0);
	vsDllExport( "int EnhProjDll_GetOfdY()", 0, 0);
	vsDllExport( "int EnhProjDll_GetOfdCX()", 0, 0);
	vsDllExport( "int EnhProjDll_GetOfdCY()", 0, 0);
	vsDllExport( "void EnhProjDll_ShowOpenFileDialog()", 0, 0);
	vsDllExport( "void EnhProjDll_StopThreads()", 0, 0);

	vsDllExport( "void FlatFile_ClearFiles()", 0, 0);
	vsDllExport( "void FlatFile_SetOptions( int sort_by_path, int case_sensitive, int always_substring_match )", 0, 0);
	vsDllExport( "void FlatFile_SetMode( int iMode )", 0, 0);
	vsDllExport( "int FlatFile_GetMode()", 0, 0);
	vsDllExport( "int FlatFile_AllocateFileList( long cFiles )", 0, 0);
	vsDllExport( "int FlatFile_AppendFile( VSHVAR filename )", 0, 0);
	vsDllExport( "void FlatFile_SortFiles()", 0, 0);
}

/**
 * You must export your functions here instead of vsDllInit if you have dll's
 * which share each other's functions.
 */
void VSAPI
vsDllRegisterExports()
{
}

/**
 * Wait for background threads to finish.  See EnhProj.e for explanation of
 * the nasty loader lock deadlock that we're trying to avoid.
 */
static void
StopThreadsInternal( bool const fWeakWait = false )
{
	AsyncShellIcon::Uninitialize( fWeakWait );
}

/**
 * vsDllExit is called when your dll is unloaded.  This function must be
 * defined even if it does nothing.
 */
void VSAPI
vsDllExit()
{
	StopThreadsInternal( true );

	FlatFile_Uninitialize();
	Str::UninitializeHeap();

	if( s_hinstRE )
	{
		FreeLibrary( s_hinstRE );
		s_hinstRE = 0;
	}

	OleUninitialize();
}

/**
 * Set the position and size for the open file dialog.
 */
void VSAPI
EnhProjDll_SetOfdPos( int x, int y, int cx, int cy )
{
	OpenDlg::SetPos( x, y, cx, cy );
}

int VSAPI EnhProjDll_IsOfdMoved()	{ return OpenDlg::IsMoved(); }
int VSAPI EnhProjDll_GetOfdX()		{ return OpenDlg::GetX(); }
int VSAPI EnhProjDll_GetOfdY()		{ return OpenDlg::GetY(); }
int VSAPI EnhProjDll_GetOfdCX()		{ return OpenDlg::GetCX(); }
int VSAPI EnhProjDll_GetOfdCY()		{ return OpenDlg::GetCY(); }

/**
 * Show the specialized open file dialog.
 */
void VSAPI
EnhProjDll_ShowOpenFileDialog()
{
	HWND const hwndFrame = reinterpret_cast<HWND>( vsPropGetL( VSWID_MDI, VSP_HWND ) );

	OpenDlg dlg;
	dlg.DoModal( hwndFrame );
}

/**
 * Stop background threads gracefully.  This should be followed by a call to
 * unload the DLL.
 */
void VSAPI
EnhProjDll_StopThreads()
{
	StopThreadsInternal();
}

EXTERN_C_END

